#!/usr/bin/env python
#
#  This file is part of Radio Tray
#
#  Copyright 2012 Costin STROIE <costinstroie@eridu.eu.org>
#
#  Radio Tray is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Radio Tray is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Radio Tray.  If not, see <http://www.gnu.org/licenses/>.
#

""" Update the bookmarks with all public stations from SKY.fm, Digitally Imported and JazzRadio """

__author__ = 'Costin Stroie'
__email__ = 'costinstroie@eridu.eu.org'
__copyright__ = 'Copyright 2012, Costin Stroie'
__license__ = 'GPLv3'
__version__ = '0.1.4'

# Import the required modules
try:
    import json
except ImportError:
    import simplejson as json
import urllib2
import os
from Plugin import Plugin
from lib.common import USER_CFG_PATH, CFG_NAME

# Radio names and base urls
RADIOS = {'sky': ('SKY.FM', 'http://listen.sky.fm/'),
          'di':  ('Digitally Imported', 'http://listen.di.fm/'),
          'jr':  ('Jazz Radio', 'http://listen.jazzradio.com/')}

# Default stream type (aac, mp3, mms)
STREAM_TYPE = 'aac'

# Main plugin class
class SkyPlugin(Plugin):
    def __init__(self):
        """ Initialize the plugin """
        super(SkyPlugin, self).__init__()

    def getName(self):
        return self.name

    def activate(self):
        self.update_stations()

    def on_menu(self, data):
        self.update_stations()

    def hasMenuItem(self):
        return True

    def update_stations(self):
        """ Update SKY.fm, Digitally Imported and Jazz Radio bookmarks """
        self.eventManagerWrapper.notify('Updating bookmarks', 'Updating SKY.fm, Digitally Imported and Jazz Radio bookmarks.')
        for radio in RADIOS:
            self.get_stations(radio)

    def write_bookmarks(self, stations, radio_name = ''):
        """ Update the bookmarks """
        from xml.dom.minidom import Document, parse

        def _remove_text_nodes(parent):
            # Remove recursively text nodes (aka beautify)
            text_nodes = [node for node in parent.childNodes if node.nodeType == parent.TEXT_NODE]
            element_nodes = [node for node in parent.childNodes if node.nodeType == parent.ELEMENT_NODE]
            for node in text_nodes:
                parent.removeChild(node)
            for node in element_nodes:
                 _remove_text_nodes(node)

        # The bookmarks file
        filename = os.path.join(USER_CFG_PATH, CFG_NAME)
        # Try to open the bookmarks file
        try:
            doc = parse(filename)
        except IOError:
            # Create a new one
            doc = Document()
        else:
            # Beautify
            _remove_text_nodes(doc)
        # The <bookmarks> element
        try:
            bookmarks = doc.getElementsByTagName('bookmarks')[0]
        except IndexError:
            bookmarks = doc.createElement('bookmarks')
            doc.appendChild(bookmarks)
        # The <group name="root"> element
        root = None
        for node in bookmarks.getElementsByTagName('group'):
            if node.getAttribute('name') == 'root':
                root = node
                break
        if root is None:
            root = doc.createElement('group')
            root.setAttribute('name', 'root')
            bookmarks.appendChild(root)
        # The <group name=""> element
        st_group = None
        for node in root.getElementsByTagName('group'):
            if node.getAttribute('name') == radio_name:
                st_group = node
                element_nodes = st_group.childNodes[:]
                for child in element_nodes:
                    st_group.removeChild(child)
                break
        if st_group is None:
            st_group = doc.createElement('group')
            st_group.setAttribute('name', radio_name)
            root.appendChild(st_group)
        # The bookmarks
        for (name, url) in stations:
            bookmark = doc.createElement('bookmark')
            bookmark.setAttribute('name', name)
            bookmark.setAttribute('url', url)
            st_group.appendChild(bookmark)
        # Save the XML
        fp = open(filename, 'wb')
        doc.writexml(fp, '', '  ', '\n')
        fp.close()

    def get_stations(self, radio, stream_type = STREAM_TYPE):
        # Create the index URL
        radio_name, radio_url = RADIOS[radio]
        if stream_type == 'aac':
            url = radio_url + 'public1'
        elif stream_type == 'mms':
            url = radio_url + 'public5'
        else:
            url = radio_url + 'public3'
        print 'Loading %s %s stations from %s' % (radio_name, stream_type, url)
        # Start with empty stations list
        stations = []
        # Parse the stations list from the server
        for station in json.load(urllib2.urlopen(url)):
            # Get the name and the playlist url
            name = station['name']
            playlist = station['playlist']
            # Append them to the list
            stations.append((name, playlist))
        # Sort the list
        stations.sort()
        # Write the bookmarks
        self.write_bookmarks(stations, radio_name = radio_name)

# vim: set ft=python ai ts=4 sts=4 et sw=4 sta nowrap nu :
